/*--------------------------------------------------------------------*\
  File:      DevBenaphore.h
  Creator:   Matt Bogosian <mbogosian@usa.net>
  Copyright: (c)1997, 1998, Matt Bogosian. All rights reserved.
  Description: Header file describing the DevBenaphore class.
  ID:        $Id: DevBenaphore.h,v 1.3 1998/07/01 05:41:11 mattb Exp $
  Conventions:
      #defines - all uppercase letters with words separated by
          underscores.
          (E.G., #define MY_DEFINE 16).
      New data types (classes, structs, typedefs, etc.) - begin with
          an uppercase letter followed by lowercase words separated by
          uppercase letters.
          (E.G., typedef int MyTypedef;).
      Global constants (declared with const) - begin with "k_"
          followed by lowercase words separated by underscores.
          (E.G., const int k_my_constant(16);).
      Global variables - begin with "g_" followed by lowercase words
          separated by underscores.
          (E.G., int g_my_global;).
      Local variables - begin with a lowercase letter followed by
          lowercase words separated by underscores.
          (E.G., int my_local;).
      Argument variables - begin with "a_" followed by lowercase words
          separated by underscores.
          (E.G., ..., int a_my_argument, ...).
      Functions - begin with a lowercase letter followed by lowercase
          words separated by uppercase letters.
          (E.G., void myFunction(void);).
      Static member constants (declared with const) - begin with "mk_"
          followed by lowercase words separated by underscores.
          (E.G., static const int MyClass::mk_my_constant(16);).
      Static member variables - begin with "mg_" followed by lowercase
          words separated by underscores.
          (E.G., static int MyClass::mg_my_global(16);).
      Member variables - begin with "m_" followed by lowercase words
          separated by underscores.
          (E.G., int m_my_member;).
      Member Functions - begin with an uppercase letter followed by
          lowercase words separated by uppercase letters.
          (E.G., void MyClass::MyFunction(void);).
\*--------------------------------------------------------------------*/


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-=-=- Included Files -=-=-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

#include <OS.h>


#ifndef DEV_BENAPHORE_H
#define DEV_BENAPHORE_H


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-= Definitions, Enums, Typedefs, Consts =-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

typedef struct _DevBenaphore *DevBenaphore;


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-=- Function Prototypes =-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

/*--------------------------------------------------------------------*\
  Function name: DevBenaphoreCreate
  Member of:     public DevBenaphore
  Defined in:    DevBenaphore.cpp
  Arguments:     DevBenaphore * const a_this - a placeholder for the
                     Benaphore to be created.
                 const char a_name[B_OS_NAME_LENGTH] - the name given
                     to the semaphore associated with the Benaphore.
  Returns:       status_t - the error code associated with the
                     operation (see below).
  Throws:        none
  Description: Class constructor. If construction should fail, the
      placeholder remains untouched. Possible return values are as
      follows:
      
      B_NO_ERROR - the object was created successfully.
      B_NO_MEMORY - there was not enough memory available to create
          the object.
      B_NO_MORE_SEMS - there were no available semaphores to create
          the object.
\*--------------------------------------------------------------------*/

status_t DevBenaphoreCreate(DevBenaphore * const a_this, const char a_name[B_OS_NAME_LENGTH]);

/*--------------------------------------------------------------------*\
  Function name: DevBenaphoreDestroy
  Member of:     public DevBenaphore
  Defined in:    DevBenaphore.cpp
  Arguments:     DevBenaphore * const a_this - the Benaphore to
                     destroy.
  Returns:       none
  Throws:        none
  Description: Class destructor.
\*--------------------------------------------------------------------*/

void DevBenaphoreDestroy(DevBenaphore * const a_this);

/*--------------------------------------------------------------------*\
  Function name: DevBenaphoreLock
  Member of:     public DevBenaphore
  Defined in:    DevBenaphore.cpp
  Arguments:     DevBenaphore a_this - the Benaphore to lock.
  Returns:       status_t - the error code associated with the
                     operation (see below).
  Throws:        none
  Description: Function to aquire a given Benaphore. If the Benaphore
      is valid, then this function will block until the Benaphore is
      locked, upon which time the function will return B_NO_ERROR. If
      the Benaphore is not valid, the function will return a value
      less than B_NO_ERROR. A successful call to DevBenaphoreLock()
      should always be mirrored by exactly one call to
      DevBenaphoreUnlock(). Possible return values are as follows:
      
      B_NO_ERROR - success.
      B_BAD_SEM_ID - the Benaphore no longer contains a valid
          semaphore.
\*--------------------------------------------------------------------*/

status_t DevBenaphoreLock(DevBenaphore a_this);

/*--------------------------------------------------------------------*\
  Function name: DevBenaphoreUnlock
  Member of:     public DevBenaphore
  Defined in:    DevBenaphore.cpp
  Arguments:     DevBenaphore a_this - the Benaphore to unlock.
  Returns:       none
  Throws:        none
  Description: Function to release a given Benaphore. There should be
      exactly one call to DevBenaphoreUnlock() for every successful
      call to DevBenaphoreLock().
\*--------------------------------------------------------------------*/

void DevBenaphoreUnlock(DevBenaphore a_this);


#endif    // DEV_BENAPHORE_H
