/*--------------------------------------------------------------------*\
  File:      ThruputView.cpp
  Creator:   Matt Bogosian <mbogosian@usa.net>
  Copyright: (c)1997, 1998, Matt Bogosian. All rights reserved.
  Description: Source file containing the member functions for the
      ThruputView class.
  ID:        $Id: ThruputView.cpp,v 1.13 1998/07/03 21:35:21 mattb Exp $
  Conventions:
      #defines - all uppercase letters with words separated by
          underscores.
          (E.G., #define MY_DEFINE 16).
      New data types (classes, structs, typedefs, etc.) - begin with
          an uppercase letter followed by lowercase words separated by
          uppercase letters.
          (E.G., typedef int MyTypedef;).
      Global constants (declared with const) - begin with "k_"
          followed by lowercase words separated by underscores.
          (E.G., const int k_my_constant(16);).
      Global variables - begin with "g_" followed by lowercase words
          separated by underscores.
          (E.G., int g_my_global;).
      Local variables - begin with a lowercase letter followed by
          lowercase words separated by underscores.
          (E.G., int my_local;).
      Argument variables - begin with "a_" followed by lowercase words
          separated by underscores.
          (E.G., ..., int a_my_argument, ...).
      Functions - begin with a lowercase letter followed by lowercase
          words separated by uppercase letters.
          (E.G., void myFunction(void);).
      Static member constants (declared with const) - begin with "mk_"
          followed by lowercase words separated by underscores.
          (E.G., static const int MyClass::mk_my_constant(16);).
      Static member variables - begin with "mg_" followed by lowercase
          words separated by underscores.
          (E.G., static int MyClass::mg_my_global(16);).
      Member variables - begin with "m_" followed by lowercase words
          separated by underscores.
          (E.G., int m_my_member;).
      Member Functions - begin with an uppercase letter followed by
          lowercase words separated by uppercase letters.
          (E.G., void MyClass::MyFunction(void);).
\*--------------------------------------------------------------------*/


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-=-=- Included Files -=-=-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

#include "ThruputView.h"

#include <Directory.h>
#include <MenuField.h>
#include <MenuItem.h>
#include <StatusBar.h>
#include <Window.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-= Definitions, Enums, Typedefs, Consts =-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

// Initialize the ReplyThread private static const data members
const uint32 ThruputView::mk_select_dev('slpt');
const uint32 ThruputView::mk_new_scale('nscl');
const uint32 ThruputView::mk_update_bps('upbd');
const float ThruputView::mk_border(5.0);
const float ThruputView::mk_hght_spc(25.0);
const float ThruputView::mk_bar_width(40.0);
const float ThruputView::mk_field_width(96.0);
const char * const ThruputView::mk_devs("Device");
const char * const ThruputView::mk_bps("Scale (bps)");
const rgb_color ThruputView::mk_high = { 0x00, 0x00, 0x80, 0x00 };
const rgb_color ThruputView::mk_light = { 0xFF - 0x10, 0xFF - 0x10, 0xFF - 0x10, 0x00 };
const rgb_color ThruputView::mk_dark = { 0xFF - 0x28, 0xFF - 0x28, 0xFF - 0x10, 0x00 };


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-= Function Definitions =-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

//====================================================================
ThruputView::ThruputView(const BRect a_frame, const char * const a_name, const port_id a_port, const char * const a_dev, const size_t a_bps, status_t * const a_err) :
//====================================================================
	ThruputView_SUPER(a_frame, a_name, B_FOLLOW_ALL, B_WILL_DRAW | B_PULSE_NEEDED),
	m_status(B_NO_ERROR),
	m_port(a_port)
{
	BRect base_rect(mk_bar_width + mk_border, 0.0, a_frame.Width() - mk_border, mk_hght_spc);
	BMenuItem *menu_item;
	BMessage *msg;
	
	// Check for an allocation error
	if ((m_devs_menu = new BMenu(mk_devs)) == NULL)
	{
		m_status = B_NO_MEMORY;
		
		// Give the error back to the caller
		if (a_err != NULL)
		{
			*a_err = Status();
		}
		
		return;
	}
	
	m_devs_menu->SetRadioMode(true);
	m_devs_menu->SetLabelFromMarked(true);
	
	// Add the devices to the menu
	char dev_path[B_PATH_NAME_LENGTH + 1], *dev_name;
	dev_path[B_PATH_NAME_LENGTH] = '\0';
	sprintf(dev_path, "/dev/ports/");
	dev_name = &dev_path[strlen(dev_path)];
	BDirectory devs_dir(dev_path);
	BEntry entry;
	
	while (devs_dir.GetNextEntry(&entry) == B_NO_ERROR)
	{
		if (entry.GetName(dev_name) == B_NO_ERROR
			&& strcmp(dev_name, "thruput") != 0)
		{
			msg = NULL;
			menu_item = NULL;
			
			// Check for an allocation error
			if ((msg = new BMessage(mk_select_dev)) == NULL
				|| (menu_item = new BMenuItem(dev_name, msg)) == NULL)
			{
				m_status = B_NO_MEMORY;
				
				// Give the error back to the caller
				if (a_err != NULL)
				{
					*a_err = Status();
				}
				
				delete m_devs_menu;
				delete msg;
				delete menu_item;
				return;
			}
			
			m_devs_menu->AddItem(menu_item);
			
			if (a_dev != NULL && strcmp(a_dev, dev_name) == 0)
			{
				menu_item->SetMarked(true);
			}
		}
	}
	
	base_rect.OffsetBy(0.0, mk_border);
	
	// Check for an allocation error
	if ((m_devs = new BMenuField(base_rect, mk_devs, mk_devs, m_devs_menu, B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP)) == NULL)
	{
		m_status = B_NO_MEMORY;
		
		// Give the error back to the caller
		if (a_err != NULL)
		{
			*a_err = Status();
		}
		
		delete m_devs_menu;
		return;
	}
	
	// Add the devices menu
	AddChild(m_devs);
	m_devs->SetEnabled(true);
	m_devs->SetAlignment(B_ALIGN_RIGHT);
	m_devs->SetDivider(mk_field_width);
	
	// Check for an allocation error
	if ((m_bps_menu = new BMenu(mk_bps)) == NULL)
	{
		m_status = B_NO_MEMORY;
		
		// Give the error back to the caller
		if (a_err != NULL)
		{
			*a_err = Status();
		}
		
		return;
	}
	
	m_bps_menu->SetRadioMode(true);
	m_bps_menu->SetLabelFromMarked(true);
	char scale_text[B_OS_NAME_LENGTH];
	
	for (size_t scale(2400); scale <= 500000; scale *= 2)
	{
		msg = NULL;
		menu_item = NULL;
		sprintf(scale_text, "%lu", scale);
		
		// Check for an allocation error
		if ((msg = new BMessage(mk_new_scale)) == NULL
			|| (menu_item = new BMenuItem(scale_text, msg)) == NULL)
		{
			m_status = B_NO_MEMORY;
			
			// Give the error back to the caller
			if (a_err != NULL)
			{
				*a_err = Status();
			}
			
			delete m_bps_menu;
			delete msg;
			delete menu_item;
			return;
		}
		
		m_bps_menu->AddItem(menu_item);
		
		if (a_bps != NULL
			&& a_bps == scale)
		{
			menu_item->SetMarked(true);
		}
		else if (m_bps_menu->FindMarked() == NULL
			&& scale >= 50000)
		{
			menu_item->SetMarked(true);
		}
	}
	
	base_rect.OffsetBy(0.0, mk_hght_spc);
	
	// Check for an allocation error
	if ((m_bps = new BMenuField(base_rect, mk_bps, mk_bps, m_bps_menu, B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP)) == NULL)
	{
		m_status = B_NO_MEMORY;
		
		// Give the error back to the caller
		if (a_err != NULL)
		{
			*a_err = Status();
		}
		
		delete m_bps_menu;
		return;
	}
	
	// Add the bps rate menu
	AddChild(m_bps);
	m_bps->SetEnabled(false);
	m_bps->SetAlignment(B_ALIGN_RIGHT);
	m_bps->SetDivider(mk_field_width);
	
	base_rect.OffsetBy(0.0, mk_hght_spc);
	base_rect.right -= (base_rect.Width() + mk_border) / 2.0;
	
	// Check for an allocation error
	if ((m_thrpt_read = new BStatusBar(base_rect, a_name, "Read", "")) == NULL)
	{
		m_status = B_NO_MEMORY;
		
		// Give the error back to the caller
		if (a_err != NULL)
		{
			*a_err = Status();
		}
		
		return;
	}
	
	m_thrpt_read->SetTrailingText("0");
	m_thrpt_read->SetMaxValue(strtoul(m_bps_menu->FindMarked()->Label(), NULL, 0));
	
	// Add the read thruput status bar
	AddChild(m_thrpt_read);
	
	base_rect.OffsetBy(base_rect.Width() + mk_border, 0.0);
	
	// Check for an allocation error
	if ((m_thrpt_write = new BStatusBar(base_rect, a_name, "Write", "")) == NULL)
	{
		m_status = B_NO_MEMORY;
		
		// Give the error back to the caller
		if (a_err != NULL)
		{
			*a_err = Status();
		}
		
		return;
	}
	
	m_thrpt_write->SetTrailingText(m_thrpt_read->TrailingText());
	m_thrpt_write->SetMaxValue(m_thrpt_read->MaxValue());
	
	// Add the write thruput status bar
	AddChild(m_thrpt_write);
	
	// Give the error back to the caller
	if (a_err != NULL)
	{
		*a_err = Status();
	}
}

//====================================================================
ThruputView::~ThruputView(void)
//====================================================================
{
	if (m_port >= B_NO_ERROR)
	{
		delete_port(m_port);
	}
}

//====================================================================
void ThruputView::AttachedToWindow(void)
//====================================================================
{
	// Set the target for the menus
	BMenuItem *item;
	
	for (size_t i(0); (item = m_devs_menu->ItemAt(i)) != NULL; i++)
	{
		item->SetTarget(this);
	}
	
	for (size_t i(0); (item = m_bps_menu->ItemAt(i)) != NULL; i++)
	{
		item->SetTarget(this);
	}
	
	// Handle the preferred device
	if (m_devs_menu->FindMarked() != NULL)
	{
		BMessage msg(mk_select_dev);
		msg.AddPointer("source", m_devs_menu->FindMarked());
		Window()->PostMessage(&msg, this);
	}
	
	// Set the colors
	SetViewColor(mk_light);
	SetLowColor(mk_dark);
	
	// Set the font
	BFont font(be_bold_font);
	font.SetFamilyAndStyle("Courier10 BT", "Bold Italic");
	font.SetFamilyAndStyle("VAGRounded BT", "Regular");
	font.SetFamilyAndStyle("CopprplGoth Bd BT", "Bold");
	font.SetFamilyAndStyle("Bazooka", "Regular");
	font.SetRotation(90.0);
	font.SetSpacing(B_BITMAP_SPACING);
	SetFont(&font);
	SetFontSize(18);
}

//====================================================================
void ThruputView::Draw(BRect /*a_update_rect*/)
//====================================================================
{
	// Draw background
	SetHighColor(mk_dark);
	FillRect(BRect(0, 0, mk_bar_width, Bounds().Height()));
	
	// Draw the title
	BFont font;
	font_height height;
	GetFont(&font);
	font.GetHeight(&height);
	MovePenTo(BPoint((mk_bar_width + height.ascent - height.descent) / 2, (StringWidth(Name()) + Bounds().Height()) / 2));
	SetHighColor(mk_high);
	DrawString(Name());
}

//====================================================================
void ThruputView::MessageReceived(BMessage * const a_msg)
//====================================================================
{
	// Handle a device selection
	if (a_msg->what == mk_select_dev
		&& m_devs->IsEnabled())
	{
		m_devs->SetEnabled(false);
		m_bps->SetEnabled(true);
		
		BMenuItem *item;
		
		if (a_msg->FindPointer("source", &item) == B_NO_ERROR)
		{
			char dev_path[B_PATH_NAME_LENGTH + 1];
			
			sprintf(dev_path, "/dev/ports/%s", item->Label());
			write_port(m_port, B_REFS_RECEIVED, dev_path, strlen(dev_path));
		}
		
		return;
	}
	// Handle the unknowns
	else if (a_msg->what != mk_new_scale
		&& a_msg->what != mk_update_bps)
	{
		ThruputView_SUPER::MessageReceived(a_msg);
		return;
	}
	
	float cur_val_read(m_thrpt_read->CurrentValue());
	float cur_val_write(m_thrpt_write->CurrentValue());
	float max_val(m_thrpt_read->MaxValue());
	
	// Handle a change of scale
	if (a_msg->what == mk_new_scale)
	{
		BMenuItem *item;
		
		if (a_msg->FindPointer("source", &item) == B_NO_ERROR)
		{
			max_val = strtoul(item->Label(), NULL, 0);
		}
	}
	// Handle a change of thruput
	else if (a_msg->what == mk_update_bps)
	{
		uint64 bps_read, bps_write;
		
		if (a_msg->FindInt64("bps read", reinterpret_cast<int64 *>(&bps_read)) == B_NO_ERROR)
		{
			cur_val_read = bps_read;
		}
		
		if (a_msg->FindInt64("bps write", reinterpret_cast<int64 *>(&bps_write)) == B_NO_ERROR)
		{
			cur_val_write = bps_write;
		}
	}
	
	// Find the appropriate colors
	rgb_color bar_color_read = { 0x00, 0x00, 0x00, 0x00 };
	rgb_color bar_color_write = { 0x00, 0x00, 0x00, 0x00 };
	
	if (cur_val_read > max_val / 2)
	{
		bar_color_read.red = 0xFF;
		bar_color_read.blue = (cur_val_read >= max_val) ? 0x00 :  (2.0 - 2.0 * cur_val_read / max_val) * 0xFF;
	}
	else
	{
		bar_color_read.red = (cur_val_read <= 0.0) ? 0x00 : 2.0 * cur_val_read / max_val * 0xFF;
		bar_color_read.blue = 0xFF;
	}
	
	if (cur_val_write > max_val / 2)
	{
		bar_color_write.red = 0xFF;
		bar_color_write.blue = (cur_val_write >= max_val) ? 0x00 :  (2.0 - 2.0 * cur_val_write / max_val) * 0xFF;
	}
	else
	{
		bar_color_write.red = (cur_val_write <= 0.0) ? 0x00 : 2.0 * cur_val_write / max_val * 0xFF;
		bar_color_write.blue = 0xFF;
	}
	
	// Update the thruput status bar (we have to update the color
	// first, because if we don't there is a weird effect that the
	// current color of the status bar will match the last
	// effective value, which is unnoticable for small changes,
	// but looks weird for big leaps)
	char text_read[B_OS_NAME_LENGTH], text_write[B_OS_NAME_LENGTH];
	
	sprintf(text_read, "%g", cur_val_read);
	sprintf(text_write, "%g", cur_val_write);
	m_thrpt_read->SetBarColor(bar_color_read);
	m_thrpt_write->SetBarColor(bar_color_write);
	m_thrpt_read->Update(cur_val_read - m_thrpt_read->CurrentValue(), NULL, text_read);
	m_thrpt_write->Update(cur_val_write - m_thrpt_write->CurrentValue(), NULL, text_write);
	m_thrpt_read->SetMaxValue(max_val);
	m_thrpt_write->SetMaxValue(max_val);
}

//====================================================================
void ThruputView::Pulse(void)
//====================================================================
{
	bigtime_t old_time;
	bigtime_t new_time;
	ssize_t msg_len;
	int32 msg_code;
	uchar buf[B_PATH_NAME_LENGTH];
	uint64 bps_read, bps_write;
	BMessage *msg;
	
	// Try to read the thruput message
	if (m_bps->IsEnabled()
		&& (msg_len = read_port(m_port, &msg_code, buf, B_PATH_NAME_LENGTH)) >= B_NO_ERROR)
	{
		switch (msg_code)
		{
			// Get the data from the thruput message
			case B_VALUE_CHANGED:
				memcpy(&bps_read, &buf[0], sizeof (bps_read));
				memcpy(&bps_write, &buf[sizeof (bps_read)], sizeof (bps_write));
				memcpy(&old_time, &buf[sizeof (bps_read) + sizeof (bps_write)], sizeof (old_time));
				new_time = real_time_clock_usecs();
				
				if (old_time == 0)
				{
					old_time = new_time;
				}
				
				// Send a message to the window with
				// this view as a target
				msg = new BMessage(mk_update_bps);
				msg->AddInt64("bps read", bps_read * 1000000 / (new_time - old_time));
				msg->AddInt64("bps write", bps_write * 1000000 / (new_time - old_time));
				Window()->PostMessage(msg, this);
				
				// Clear the message and write it back
				// to the port
				bps_read = bps_write = 0;
				old_time = new_time;
				memcpy(&buf[0], &bps_read, sizeof (bps_read));
				memcpy(&buf[sizeof (bps_read)], &bps_write, sizeof (bps_write));
				memcpy(&buf[sizeof (bps_read) + sizeof (bps_write)], &old_time, sizeof (old_time));
				write_port(m_port, B_VALUE_CHANGED, buf, sizeof (bps_read) + sizeof (bps_write) + sizeof (old_time));
				break;
			
			// Write any others straight back to the port
			default:
				write_port(m_port, msg_code, buf, msg_len);
				break;
		}
	}
	
	// Quit if the port's dead
	if (msg_len == B_BAD_PORT_ID)
	{
		m_port = msg_len;
		Window()->PostMessage(B_QUIT_REQUESTED);
	}
}

//====================================================================
const char *ThruputView::SelectedDevice(void)
//====================================================================
{
	if (m_devs_menu->FindMarked() != NULL)
	{
		return m_devs_menu->FindMarked()->Label();
	}
	
	return NULL;
}

//====================================================================
size_t ThruputView::SelectedScale(void)
//====================================================================
{
	return strtoul(m_bps_menu->FindMarked()->Label(), NULL, 0);
}
