/*--------------------------------------------------------------------*\
  File:      ThruputWindow.cpp
  Creator:   Matt Bogosian <mbogosian@usa.net>
  Copyright: (c)1997, 1998, Matt Bogosian. All rights reserved.
  Description: Source file containing the member functions for the
      ThruputWindow class.
  ID:        $Id: ThruputWindow.cpp,v 1.13 1998/07/06 09:24:14 mattb Exp $
  Conventions:
      #defines - all uppercase letters with words separated by
          underscores.
          (E.G., #define MY_DEFINE 16).
      New data types (classes, structs, typedefs, etc.) - begin with
          an uppercase letter followed by lowercase words separated by
          uppercase letters.
          (E.G., typedef int MyTypedef;).
      Global constants (declared with const) - begin with "k_"
          followed by lowercase words separated by underscores.
          (E.G., const int k_my_constant(16);).
      Global variables - begin with "g_" followed by lowercase words
          separated by underscores.
          (E.G., int g_my_global;).
      Local variables - begin with a lowercase letter followed by
          lowercase words separated by underscores.
          (E.G., int my_local;).
      Argument variables - begin with "a_" followed by lowercase words
          separated by underscores.
          (E.G., ..., int a_my_argument, ...).
      Functions - begin with a lowercase letter followed by lowercase
          words separated by uppercase letters.
          (E.G., void myFunction(void);).
      Static member constants (declared with const) - begin with "mk_"
          followed by lowercase words separated by underscores.
          (E.G., static const int MyClass::mk_my_constant(16);).
      Static member variables - begin with "mg_" followed by lowercase
          words separated by underscores.
          (E.G., static int MyClass::mg_my_global(16);).
      Member variables - begin with "m_" followed by lowercase words
          separated by underscores.
          (E.G., int m_my_member;).
      Member Functions - begin with an uppercase letter followed by
          lowercase words separated by uppercase letters.
          (E.G., void MyClass::MyFunction(void);).
\*--------------------------------------------------------------------*/


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-=-=- Included Files -=-=-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

#include "ThruputWindow.h"
#include "ThruputView.h"

#include "AddOn.h"
#include "PMain.h"


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-= Function Definitions =-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

//====================================================================
ThruputWindow::ThruputWindow(const port_id a_port, AddOn * const a_add_on, status_t * const a_err) :
//====================================================================
	ThruputWindow_SUPER(BRect(0, 0, 224, 93), "Thruput", B_MODAL_WINDOW_LOOK, B_NORMAL_WINDOW_FEEL, B_NOT_CLOSABLE | B_NOT_ZOOMABLE | B_NOT_RESIZABLE),
	m_add_on(a_add_on)
{
	// Try to read the preferences
	size_t size;
	type_code type;
	BPoint real_pos(160, 120);
	char *real_dev(NULL);
	size_t real_bps(0);
	BPoint pref_pos;
	char pref_dev[B_FILE_NAME_LENGTH + 1];
	size_t pref_bps;
	
	if (!(modifiers() & B_CAPS_LOCK
		|| modifiers() & B_SHIFT_KEY))
	{
		size = sizeof (pref_pos);
		
		if (m_add_on->SettingRead("pos", &pref_pos, &size, &type) == B_NO_ERROR
			&& type == B_POINT_TYPE)
		{
			real_pos = pref_pos;
		}
		
		pref_dev[size = B_FILE_NAME_LENGTH] = '\0';
		
		if (m_add_on->SettingRead("dev", &pref_dev, &size, &type) == B_NO_ERROR
			&& type == B_STRING_TYPE)
		{
			real_dev = pref_dev;
		}
		
		size = sizeof (pref_bps);
		
		if (m_add_on->SettingRead("bps", &pref_bps, &size, &type) == B_NO_ERROR
			&& type == B_INT32_TYPE)
		{
			real_bps = pref_bps;
		}
	}
	
	// Set up the window
	SetPulseRate(200000);
	MoveTo(real_pos);
	
	// Set up and instantiate the main view
	m_status = B_NO_MEMORY;
	BRect frame(Bounds());
	frame.OffsetTo(B_ORIGIN);
	
	// Check for an allocation error
	if ((m_main_view = new ThruputView(frame, Title(), a_port, real_dev, real_bps, &m_status)) == NULL
		|| m_status != B_NO_ERROR)
	{
		delete m_main_view;
		
		// Give the error back to the caller
		if (a_err != NULL)
		{
			*a_err = Status();
		}
		
		return;
	}
	
	AddChild(m_main_view);
	
	// Try to add ourselves to the add-on
	m_status = m_add_on->AddLooper(this);
	
	// Give the error back to the caller
	if (a_err != NULL)
	{
		*a_err = Status();
	}
}

//====================================================================
ThruputWindow::~ThruputWindow(void)
//====================================================================
{
	;
}

//====================================================================
bool ThruputWindow::QuitRequested(void)
//====================================================================
{
	// Save our settings
	if (modifiers() & B_CAPS_LOCK
		|| modifiers() & B_SHIFT_KEY)
	{
		BPoint pos(Frame().LeftTop());
		m_add_on->SettingWrite("pos", &pos, sizeof (pos), B_POINT_TYPE);
		
		if (m_main_view->SelectedDevice() != NULL)
		{
			m_add_on->SettingWrite("dev", m_main_view->SelectedDevice(), strlen(m_main_view->SelectedDevice()) + 1, B_STRING_TYPE);
		}
		
		size_t bps(m_main_view->SelectedScale());
		m_add_on->SettingWrite("bps", &bps, sizeof (bps), B_INT32_TYPE);
	}
	
	// Let the dev_server clean us up (that's why we return
	// false). We could return true here, but we would have to
	// remove ourselves from the AddOn to keep it from calling the
	// Lock() and Quit() member functions to this BLooper. In
	// other words, we don't want it to try to Quit() a BLooper
	// that's already been deleted, which is what it would do if
	// we returned true here. Instead, we just stop Pulse()ing (to
	// avoid any more quit requests from our view), and request
	// that we be cleaned up by the dev_server itself. Maybe I
	// just like to be dominated...who knows?
	SetPulseRate(0);
	m_add_on->RequestKill();
	
	return false;
}
